/**
 * \file navigator.hpp
 *
 * Header file for the rednav navigator tool.
 *
 * \date 2011-01-07
 * \author redouane.boumghar@laas.fr
 * \ingroup rednav
 */


#ifndef REDNAV_LIBS_NAVIGATOR_HPP
#define REDNAV_LIBS_NAVIGATOR_HPP

// For the definition of jafar::lgl::NavGraph::NavPath
#include <rednav/pathTool.hpp>
#include <lgl/NavGraph.hpp>
#include <lgl/GeoData.hpp>
#include <lgl/Location.hpp>
#include <iostream>
//#include <fstream>


namespace jafar {
namespace rednav {

	class Navigator {
		private:
			/** Map x size */
			int xsize;
			/** Map y size */
			int ysize;
			/** Map y size */
			int zsize;
			/**  Map X Origin in the ENU utm frame */
			double xMapOrigin;
			/**  Map Y Origin in the ENU utm frame */
			double yMapOrigin;
			/**  Map Z Origin in the ENU utm frame */
			double zMapOrigin;
			/**  Map pixels scale number of meter(s) for one pixel */
			double pixelScale;
			/** X Start coordinates in the position manager frame */
			int startx;
			/** Y Start coordinates in the position manager frame */
			int starty;
			/** Z Start coordinates in the position manager frame */
			int startz;
			/** X Start coordinates in the ENU UTM Frame */
			double startutmx;
			/** Y Start coordinates in the ENU UTM Frame */
			double startutmy;
			/** Z Start coordinates in the ENU UTM Frame */
			double startutmz;
			/** X Goal coordinates in the position manager frame */
			int goalx;
			/** Y Goal coordinates in the position manager frame */
			int goaly;
			/** Z Goal coordinates in the position manager frame */
			int goalz;
			/** X Goal coordinates in the ENU UTM Frame */
			double goalutmx;
			/** Y Goal coordinates in the ENU UTM Frame */
			double goalutmy;
			/** Z Goal coordinates in the ENU UTM Frame */
			double goalutmz;

			/** Navigation Map */
			jafar::lgl::GeoData *navMap;

			/** Map quad decomposition */
			jafar::lgl::Quadtree<double, jafar::lgl::GeoData> *navQuadtree;

			/** Navigation LGL graph */
			jafar::lgl::NavGraph *navGraph;

			PathTool *multiTool;

		public:
			/** Default constructor */
			Navigator();
			/** Constructor with the map as input */
			Navigator(double* bufferedData, int xsize_, int ysize_, int zsize_, double xMapOrigin_, double yMapOrigin_, double zMapOrigin_, double pixelScale_);

			/** Destructor */
			~Navigator();

			/** Save geodata and quadtree */
			bool saveData(const std::string& outfile, const std::string& quadTreeFile);

			/** Find a nominal path */
			bool findPath(const jafar::lgl::NavLocation& locstart, const jafar::lgl::NavLocation& locend, jafar::lgl::NavPath& navpath);

			/** Update map and recreate quadtree and navigation graph accordingly */
			bool updateMap(double* bufferedData, int xsize_, int ysize_);
			/** Setting map origin and resolution */
			void setMapOrigin(double xMapOrigin_, double yMapOrigin_, double zMapOrigin_, double pixelScale_);
			/** Set the map size */
			void setMapSize(int  xsize_, int ysize_, int zsize_);
			/** Setting the goal through the position manager frame */
			void setGoal(int x, int y, int z);
			/** Setting the goal through the ENU(utm) frame */
			void setGoal(double x, double y, double z);
			/** Setting the goal through the position manager frame */
			void setStart(int x, int y, int z);
			/** Setting the goal through the ENU(utm) frame */
			void setStart(double x, double y, double z);

	};
}} // end of namespaces

#endif //REDNAV_LIBS_NAVIGATOR_HPP
