/**
 * \file navigator.cpp
 *
 * Source file for the navigator tool.
 *
 * \date 2011-01-09
 * \author redouane.boumghar@laas.fr
 * \ingroup rednav
 */

#include <rednav/navigator.hpp>

namespace jafar {
namespace rednav {

	// Constructor that does nothing but giving calls to tools
	Navigator::Navigator()
	{ // {{{
	} // }}}

	// Tooled Constructor
	Navigator::Navigator(double* bufferedData, int xsize_, int ysize_, int zsize_, double xMapOrigin_, double yMapOrigin_, double zMapOrigin_, double pixelScale_)
	{ //{{{
		setMapOrigin(xMapOrigin_, yMapOrigin_, zMapOrigin_, pixelScale_);
		setMapSize(xsize_, ysize_, zsize_);

		navMap = new jafar::lgl::GeoData(xsize, ysize, bufferedData);
		navQuadtree = new jafar::lgl::Quadtree<double, jafar::lgl::GeoData>(navMap);
		navGraph = new jafar::lgl::NavGraph(navMap, navQuadtree);
		multiTool = new PathTool();
	} //}}}

	//-------------------------- Destructor ---------
	Navigator::~Navigator()
	{ // {{{
		delete navMap;
		delete navQuadtree;
		delete navGraph;
	} // }}}

	bool Navigator::saveData(const std::string& outfile, const std::string& quadTreeFile)
	{/*{{{*/
		navMap->saveData(outfile);
		std::ofstream quadTreeStream (quadTreeFile.c_str());
		quadTreeStream << (*navQuadtree);
		quadTreeStream.close();
		return true;
	}/*}}}*/

	bool Navigator::findPath(const jafar::lgl::NavLocation& locstart, const jafar::lgl::NavLocation& locend, jafar::lgl::NavPath& navpath)
	{//{{{
		return navGraph->findPath(locstart, locend, navpath);
	}//}}}

	bool Navigator::updateMap(double* bufferedData, int xsize_, int ysize_)
	{//{{{
		delete navGraph;
		navQuadtree = NULL;
		setMapSize(xsize_, ysize_, zsize);
		if (navMap->setData(bufferedData, xsize, ysize))
		{
			// Recreating quadtree
			navQuadtree = new jafar::lgl::Quadtree<double, jafar::lgl::GeoData>(navMap);
			// Recreating navgraph
			navGraph = new jafar::lgl::NavGraph(navMap, navQuadtree);
			return true;
		}
		return false;
	}//}}}

	void Navigator::setMapOrigin(double xMapOrigin_, double yMapOrigin_, double zMapOrigin_, double pixelScale_)
	{//{{{
		xMapOrigin = xMapOrigin_;
		yMapOrigin = yMapOrigin_;
		zMapOrigin = zMapOrigin_;
		pixelScale = pixelScale_;
	}//}}}

	void Navigator::setMapSize(int  xsize_, int ysize_, int zsize_)
	{//{{{
		xsize = xsize_;
		ysize = ysize_;
		zsize = zsize_;
	}//}}}

	void Navigator::setGoal(int x, int y, int z)
	{//{{{
		goalx = x;
		goaly = y;
		goalz = z;
	}//}}}

	void Navigator::setGoal(double x, double y, double z)
	{//{{{
		goalx = x;
		goaly = y;
		goalz = z;
	}//}}}

	void Navigator::setStart(int x, int y, int z)
	{//{{{
		startx = x;
		starty = y;
		startz = z;
	}//}}}

	void Navigator::setStart(double x, double y, double z)
	{//{{{
		startx = x;
		starty = y;
		startz = z;
	}//}}}

}} // end of namespaces
